@@ -24,6 +24,10 @@
 
 #include "fastGHQuad.h"
 
+#ifndef FCONE
+# define FCONE
+#endif
+
 using std::vector;
 using std::abs;
 
@@ -107,7 +111,7 @@ void quadInfoGolubWelsch(int n, vector<double> &D, vector<double> &E,
   F77_NAME(dstev)(&JOBZ, &n, &D[0], &E[0],  // Job flag & input matrix
                   &Z[0], &n,       // Output array for eigenvectors & dim
                   &WORK[0], &INFO  // Workspace & info flag
-                  );
+                  FCONE);
 
   // Setup x & w
   int i;
@@ -172,7 +176,7 @@ void findPolyRoots(const vector<double> &c, int n, vector<double> *r) {
       &tmpwork,            // Workspace; will contain optimal size upon exit
       &LWORK,              // Workspace size; -1 -> get optimal size
       &INFO                // Status code
-      );
+      FCONE FCONE);
 
   // Next, actually run eigendecomposition
   LWORK = (int)tmpwork;
@@ -186,7 +190,7 @@ void findPolyRoots(const vector<double> &c, int n, vector<double> *r) {
       &work[0],            // Workspace; will contain optimal size upon exit
       &LWORK,              // Workspace size; -1 -> get optimal size
       &INFO                // Status code
-      );
+      FCONE FCONE);
 }
 
 SEXP findPolyRoots(SEXP cR) {
